from __future__ import print_function

import numpy as np
import tensorflow as tf
import tensorflow.keras.layers as klayers
import random
#import collections
import os
import time
import pickle
import datetime as dt


# os.environ["CUDA_DEVICE_ORDER"] ="PCI_BUS_ID"
# os.environ["CUDA_VISIBLE_DEVICES"] ="0"
#
#
# config = tf.compat.v1.ConfigProto()
# config.gpu_options.allow_growth = True
# tf.compat.v1.keras.backend.set_session(tf.compat.v1.Session(config=config))


corpus_file='lm_corpus.pkl'
dict_file ='lm_dict.pkl'
embed_file ='lmr_embeddings.pkl'

###############################################
# Load the pickle files and process data
################################################
def build_dataset():

    with open(corpus_file, 'rb') as f:
        lm_corpus=pickle.load(f)

    with open(dict_file, 'rb') as f:
        lm_dict = pickle.load(f)

    with open(embed_file, 'rb') as f:
        lm_embeds = pickle.load(f)

    reverse_lm_dict = dict(zip(lm_dict.values(), lm_dict.keys()))

    return lm_corpus, lm_dict, reverse_lm_dict, lm_embeds



########################################################
# Build dataset by reading the files saved in word embedding part
########################################################
lmc, lmd, rlmd, lme_r=build_dataset()
#lme_r=reduce_dim(lme, reduced_embedding_dim)
training_data =np.array(lmc)
dictionary=lmd
reverse_dictionary=rlmd

vocab_size = len(lmd)


##############################
# Parameters
############################
embedding_dim=5

learning_rate = 0.001
training_iters = 100
time_steps = 4
input_fea_dim= 1
batch_size=3


######################################################################################
# Define the data generator
#####################################################################################
def generate_data(input_fea_dim,vocab_size, time_steps, training_data, dictionary,lme_r):

    offset = random.randint(0, time_steps + 1)
    end_offset = time_steps + 1

    while 1:
        # Generate a minibatch. Add some randomness on selection process.
        x_batch=[]
        y_batch=[]

        for bs in range (batch_size):
            if offset > (len(training_data)-end_offset):
                offset = random.randint(0, time_steps+1)

            # values for populating input  x
            symbols_in_keys = [ [dictionary[ str(training_data[i])]] for i in range(offset, offset+time_steps) ]
            if (input_fea_dim==1):
                symbols_in_keys = np.reshape(np.array(symbols_in_keys), [-1, time_steps, input_fea_dim])
            else:
                symbols_in_keys= np.reshape(lme_r[symbols_in_keys,:], [-1, time_steps, input_fea_dim])

            #values for populating one hot encoded label  y
            symbols_out_onehot = np.zeros([vocab_size], dtype=float)
            symbols_out_onehot[dictionary[str(training_data[offset+time_steps])]] = 1.0
            symbols_out_onehot = np.reshape(symbols_out_onehot,[1,-1])

            x_batch.append(symbols_in_keys)
            y_batch.append(symbols_out_onehot)

            offset += (time_steps + 1)

        x_batch_=np.concatenate(x_batch, axis=0)
        y_batch_ = np.concatenate(y_batch, axis=0)

        #yield the current batch
        yield(x_batch_, y_batch_)



##########################################
# number of units in RNN cell and cell type
##########################################
n_hidden = 512



############################################
#  Define the model
##############################################
def RNN(indim,n_hidden, outdim):

    input=klayers.Input(shape=(None,indim))

    hidden=klayers.LSTM(n_hidden, name='lstm')(input)

    output=klayers.Dense(outdim, activation=None)(hidden)

    return tf.keras.Model(input, output)


##############################################################
# Create the RNN model
#############################################################
rnn_model= RNN(input_fea_dim, n_hidden, vocab_size)

######################################
# Loss and optimizer
######################################
def loss(labels,logits):
    return tf.keras.losses.categorical_crossentropy(labels, logits, from_logits=True)

optimizer=tf.keras.optimizers.RMSprop(learning_rate=learning_rate)

##########################################
# Compile the model
##########################################
rnn_model.compile(optimizer=optimizer, loss=loss)


##########################################################
# Fit the model to data
##########################################################
rnn_model.fit_generator(generate_data(input_fea_dim,vocab_size, time_steps, training_data, dictionary,lme_r), steps_per_epoch=50, epochs=training_iters)


######################################################
# Evaluate
######################################################
for offset in range(1,len(training_data)-time_steps,10 ):

    # values for populating  x
    symbols_in_keys = [[dictionary[str(training_data[i])]] for i in range(offset, offset + time_steps)]
    if (input_fea_dim == 1):
        symbols_in_keys = (np.reshape(np.array(symbols_in_keys), [-1, time_steps, input_fea_dim])).astype(float)
    else:
        symbols_in_keys = np.reshape(lme_r[symbols_in_keys, :], [-1, time_steps, input_fea_dim])

    # Make the prediction
    onehot_pred=rnn_model.predict(symbols_in_keys)

    # Print the result
    symbols_in = [training_data[i] for i in range(offset, offset + time_steps)]
    symbols_out = training_data[offset + time_steps]
    symbols_out_pred = reverse_dictionary[int(np.argmax(onehot_pred, 1))]
    print("%s - [%s] vs [%s]" % (symbols_in,symbols_out,symbols_out_pred))







